=begin
(c) TIG 2010
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________
Script:
  ####hideContextInstances.rb
place it in the ../Plugins/ folder...
___________________________________________________________________________
Usage:
This script adds a new function to the 'View' menu.
  'Editing: Show Geometry Only'
It 'toggles'.
If it is active it is shown 'ticked'.
If it is inactive it is shown 'unticked'.
The default setting is 'inactive'.
The last setting is remembered with the saved model across sessions.
When it is 'active' if you edit a component-instance or a group then 
every group/instance/image within its entities will be hidden from view - 
only its raw geometry will be visible.
This may be more useful than 'Hide Rest of Model' [or similar tools] if 
there are groups/instances/images that might obscure your view while 
editing the instance's raw geometry...
Note that if the edited instance's entities contains no raw geometry at all 
then there will be an 'empty' edit session.
Thus nested instances/groups will not be visible and therefore not 
selectable, unless you toggle 'View'>'Hidden-Geometry' to be 'on'.
Then you can select 'hidden' instances/groups...
Set 'View'>'Hidden Geometry' to 'off' for an unobstructed view 
otherwise.
Instances/groups in the current active_entities are only 'hidden' as you 
edit 'down' through any nesting - as you 'open' each instance to edit.  
As you edit back 'up' through the nesting they will then remain visible - 
to hide them again you must redo the edit 'down' again from scratch...
Note that because it is a menu-item you can shortcut-key to it to 'toggle'.
___________________________________________________________________________
Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________
Version:
1.0 20101020 First Release.
1.1 20101020 Toggle now works both ways within an edit.
___________________________________________________________________________
=end
#####################
require 'sketchup.rb'
#####################
### extra definition methods for group/image
class Sketchup::Group
  def definition
    if self.entities.parent.instances.include?(self)
      return self.entities.parent
    else
      Sketchup.active_model.definitions.each{|definition|
        return definition if definition.instances.include?(self)
      }
    end#if
    return nil
  end
end
#####
class GeomOnlyRestHiddenInstanceObserver < Sketchup::InstanceObserver
  def onOpen(instance)
    model=Sketchup.active_model
    if model.get_attribute("GeomOnlyRestHidden","toggle","")==""
      model.set_attribute("GeomOnlyRestHidden","toggle","off")
    end#if
    $geomOnlyRestHiddenArray.each{|e|e.hidden=false if e.valid?} if $geomOnlyRestHiddenArray
    $geomOnlyRestHiddenArray=[]
    if model.get_attribute("GeomOnlyRestHidden","toggle","")=="on"
      entsa=instance.definition.entities.to_a
      entsa.each{|e|
        if e.valid? and not e.hidden? and (e.class==Sketchup::Group or e.class==Sketchup::ComponentInstance or e.class==Sketchup::Image)
          $geomOnlyRestHiddenArray << e
          e.hidden=true
        end#if
      }
      begin
        instance.parent.remove_observer($geomOnlyRestHiddenInstanceObserver) if instance.parent != model
      rescue
        ###
      end
      instance.parent.remove_observer($geomOnlyRestHiddenInstanceObserver) if instance.parent != model
    end#if
    model.active_view.invalidate
  end#def
  ###
  def onClose(instance)
    model=Sketchup.active_model
    $geomOnlyRestHiddenArray.each{|e|e.hidden=false if e.valid?} if $geomOnlyRestHiddenArray
    $geomOnlyRestHiddenArray=[]
    model.active_view.invalidate
  end#def
end#class
###########################################################################
$geomOnlyRestHiddenInstanceObserver=GeomOnlyRestHiddenInstanceObserver.new
###########################################################################

### update as defs / instances added
class GeomOnlyRestHiddenDefinitionsObserver < Sketchup::DefinitionsObserver
  def onComponentAdded(definitions, definition)
    definitions.each{|d|
      begin
        d.entities.remove_observer($geomOnlyRestHiddenEntitiesObserver)
      rescue
        ###
      end
      d.entities.add_observer($geomOnlyRestHiddenEntitiesObserver)
      ###
      d.instances.each{|i|
        begin
          i.remove_observer($geomOnlyRestHiddenInstanceObserver)
        rescue
          ###
        end
        i.add_observer($geomOnlyRestHiddenInstanceObserver)
      }
    }
  end#def
  def onComponentTypeChanged(definitions, definition)
    definitions.each{|d|
      begin
        d.entities.remove_observer($geomOnlyRestHiddenEntitiesObserver)
      rescue
        ###
      end
      d.entities.add_observer($geomOnlyRestHiddenEntitiesObserver)
      ###
      d.instances.each{|i|
        begin
          i.remove_observer($geomOnlyRestHiddenInstanceObserver)
        rescue
          ###
        end
        i.add_observer($geomOnlyRestHiddenInstanceObserver)
      }
    }
  end#def
end#class
###########################################################################
$geomOnlyRestHiddenDefinitionsObserver=GeomOnlyRestHiddenDefinitionsObserver.new
###########################################################################

### model observer
class GeomOnlyRestHiddenModelObserver < Sketchup::ModelObserver
  #def onActivePathChanged(model)
    #$geomOnlyRestHiddenArray.each{|e|e.hidden=false if e.valid?} if $geomOnlyRestHiddenArray
  #end
  def onPreSaveModel(model)
    $geomOnlyRestHiddenArray.each{|e|e.hidden=false if e.valid?} if $geomOnlyRestHiddenArray
  end
  def onSaveModel(model)
    $geomOnlyRestHiddenArray.each{|e|e.hidden=true if e.valid?} if $geomOnlyRestHiddenArray
  end
end
###########################################################################
$geomOnlyRestHiddenModelObserver=GeomOnlyRestHiddenModelObserver.new
###########################################################################

### observe for added instances because DefinitionObserver is FUBAR on making 'copies' !!!
class GeomOnlyRestHiddenEntitiesObserver < Sketchup::EntitiesObserver
  def onElementAdded(entities, entity)
    model=Sketchup.active_model
    if entity.class==Sketchup::Group or entity.class==Sketchup::ComponentInstance or entity.class==Sketchup::Image
      begin
        entity.remove_observer($geomOnlyRestHiddenInstanceObserver)
      rescue
        ###
      end
      entity.add_observer($geomOnlyRestHiddenInstanceObserver)
    end#if
  end
end#class
###########################################################################
$geomOnlyRestHiddenEntitiesObserver=GeomOnlyRestHiddenEntitiesObserver.new
###########################################################################


### we must load observers at start #######################################
#######--------------------------------------------------------------------
def addGeomOnlyRestHiddenObserver()
  model=Sketchup.active_model
  if model.get_attribute("GeomOnlyRestHidden","toggle","")==""
    model.set_attribute("GeomOnlyRestHidden","toggle","off")
  end#if
  if model.get_attribute("GeomOnlyRestHidden","toggle","")=="on"
    begin
      model.definitions.remove_observer($geomOnlyRestHiddenDefinitionsObserver)
    rescue
      ###
    end
    model.definitions.add_observer($geomOnlyRestHiddenDefinitionsObserver)
    ###
    model.definitions.each{|d|
      begin
        d.entities.remove_observer($geomOnlyRestHiddenEntitiesObserver)
      rescue
        ###
      end
      d.entities.add_observer($geomOnlyRestHiddenEntitiesObserver)
      ###
      d.instances.each{|i|
        begin
          i.entities.remove_observer($geomOnlyRestHiddenInstanceObserver)
        rescue
          ###
        end
        i.add_observer($geomOnlyRestHiddenInstanceObserver)
      }
    }
    begin
      model.entities.remove_observer($geomOnlyRestHiddenEntitiesObserver)
    rescue
      ###
    end
    model.entities.add_observer($geomOnlyRestHiddenEntitiesObserver)
    ###
    begin
      model.remove_observer($geomOnlyRestHiddenModelObserver)
    rescue
      ###
    end
    model.add_observer($geomOnlyRestHiddenModelObserver)
  end#
end#def
####### run at load
addGeomOnlyRestHiddenObserver()
#######--------------------------------------------------------------------

## remove observers #######################################
#######--------------------------------------------------------------------
def removeGeomOnlyRestHiddenObserver()
  model=Sketchup.active_model
  if model.get_attribute("GeomOnlyRestHidden","toggle","")==""
    model.set_attribute("GeomOnlyRestHidden","toggle","off")
  end#if
  if model.get_attribute("GeomOnlyRestHidden","toggle","")=="off"
    begin
      model.definitions.remove_observer($geomOnlyRestHiddenDefinitionsObserver)
    rescue
      ###
    end
    ###
    model.definitions.each{|d|
      begin
        d.entities.remove_observer($geomOnlyRestHiddenEntitiesObserver)
      rescue
        ###
      end
      ###
      d.instances.each{|i|
        begin
          i.entities.remove_observer($geomOnlyRestHiddenInstanceObserver)
        rescue
          ###
        end
      }
    }
    begin
      model.entities.remove_observer($geomOnlyRestHiddenEntitiesObserver)
    rescue
      ###
    end
    ###
    begin
      model.remove_observer($geomOnlyRestHiddenModelObserver)
    rescue
      ###
    end
  end#
end#def
#######--------------------------------------------------------------------

#### also ensure it works on new/menu models etc with an AppObserver
class GeomOnlyRestHiddenAppObserver < Sketchup::AppObserver
  def onNewModel(model)  ### from menu or app
    addGeomOnlyRestHiddenObserver()
  end
  def onOpenModel(model) ### from menu
    addGeomOnlyRestHiddenObserver()
  end
  def onQuit()
    $geomOnlyRestHiddenArray.each{|e|e.hidden=false if e.valid?} if $geomOnlyRestHiddenArray
  end
end
# Attach the observer at kick-off
Sketchup.add_observer(GeomOnlyRestHiddenAppObserver.new)
########################################################

###########################################################################
#### tool ##############
def GeomOnlyRestHidden()
  model=Sketchup.active_model
  defs=model.definitions
  if model.get_attribute("GeomOnlyRestHidden","toggle","")==""
    model.set_attribute("GeomOnlyRestHidden","toggle","off")
  end#if
  if model.get_attribute("GeomOnlyRestHidden","toggle","")=="off"
    model.set_attribute("GeomOnlyRestHidden","toggle","on")
    addGeomOnlyRestHiddenObserver() ###
    if model.active_entities != model.entities
      entsa=model.active_entities.to_a
      entsa.each{|e|
        if e.valid? and not e.hidden? and (e.class==Sketchup::Group or e.class==Sketchup::ComponentInstance or e.class==Sketchup::Image)
          $geomOnlyRestHiddenArray << e
          e.hidden=true
        end#if
      }
    end#if
  else ### undo anything current
    model.set_attribute("GeomOnlyRestHidden","toggle","off")
    removeGeomOnlyRestHiddenObserver() ###
    if model.active_entities != model.entities
      $geomOnlyRestHiddenArray.each{|e|e.hidden=false if e.valid?} if $geomOnlyRestHiddenArray
      $geomOnlyRestHiddenArray=[]
    end#if
  end#if
end#def
############################################################

###########################################################################
### menu
if not file_loaded?(File.basename(__FILE__))
  menu=UI.menu("View").add_item("Editing: Show Geometry Only"){GeomOnlyRestHidden()}
  UI.menu.set_validation_proc(menu){
    if Sketchup.active_model.get_attribute("GeomOnlyRestHidden","toggle","")=="off"
      MF_UNCHECKED
    else
      MF_CHECKED
    end#if
  }
end
file_loaded(File.basename(__FILE__))
###########################################################################